

import calendar
import subprocess
import laptops_final
from laptops_final import (
    average_laptops_checked_out,
    plot_average_loans_by_day_of_week,
    plot_monthly_loan_trend
)


#get_ipython().system('pip install flask')


from flask import Flask, request, jsonify, render_template_string, render_template
import threading


app = Flask(__name__,template_folder="html")


historical_results = []






# Routes
@app.route('/')
def index():
    return render_template("welcome.html")

@app.route('/redirect_to_calculation', methods=['POST'])
def redirect_to_calculation():
    option = request.form['option']
    
    if option == 'calculate_average':
        return render_template("html.html")
    elif option == 'plot_trend':
        return render_template("graph_input.html")
    elif option == 'average_loans_day_of_week':
        return render_template("average_loans.html")

# Store historical results
historical_results = []

@app.route('/calculate_average', methods=['POST'])
def calculate_average():
    start_date = request.form['start_date']
    end_date = request.form['end_date']
    
    try:
        # Call your function to calculate the result
        result = average_laptops_checked_out(start_date, end_date)
        result=float(result)
        result=round(result)
        # Append the result to history
        historical_results.append({
            'start_date': start_date,
            'end_date': end_date,
            'result': result
        })
        
        # Render the template with result and history
        return render_template("html.html", result=result, history=historical_results)
    except Exception as e:
        return render_template("html.html", result=f"Error: {str(e)}", history=historical_results)


@app.route('/average_loans_day_of_week', methods=['POST'])
def average_loans_day_of_week():
    start_date = request.form['start_date']
    end_date = request.form['end_date']
    
    try:
        # Call the function and generate the plot
        plot_path = plot_average_loans_by_day_of_week(start_date, end_date)
        
        # Return the HTML with the image displayed
        return render_template("average_loans.html", plot_path=plot_path)
    except Exception as e:
        return f"<h1>Error: {str(e)}</h1>"

@app.route('/plot_trend', methods=['POST'])
def plot_trend():
    start_date=request.form['start_month']
    end_date=request.form['end_month']
    year, month = map(int, end_date.split('-'))
    last_day = calendar.monthrange(year, month)[1]
    start_date=start_date+'-01'
    end_date=end_date+'-'+str(last_day)
    print(start_date,end_date)
    try:
        # Call your function to get the plot and data
        plot_path, monthly_loans = plot_monthly_loan_trend(start_date, end_date)
        
       # Convert the 'Month-Year' column to string format without time
        monthly_loans['Month-Year'] = monthly_loans['Month-Year'].dt.strftime('%Y-%m')

        # Convert the DataFrame to a list of dictionaries for rendering
        monthly_loans_data = monthly_loans.to_dict(orient='records')

        # Debugging: Print data to check correctness
        print("Plot Path:", plot_path)
        print("Monthly Loans Data:", monthly_loans_data)
        
        # Render the HTML template with the plot and data
        return render_template("monthly_loans.html", plot_path=plot_path, monthly_loans=monthly_loans_data)
    except Exception as e:
        return f"<h1>Error: {str(e)}</h1>"




@app.route('/run_script', methods=['POST'])
def run_script():
    # Retrieve the password submitted from the form.
    user_password = request.form.get("password", "").strip()

    # Read the correct password from password.docx.
    # (Assuming password.docx contains a plain-text password)
    try:
        with open("password.txt", "r", encoding="utf-8") as f:
            correct_password = f.read().strip()
    except Exception as e:
        message = f"Error reading password file: {e}"
        return render_template("welcome.html", message=message, success=False)

    # Verify the entered password.
    if user_password != correct_password:
        message = "Incorrect password."
        return render_template("welcome.html", message=message, success=False)

    # If the password is correct, run the subprocess.
    try:
        completed_process = subprocess.run(
            ["python", "laptops_final.py"],
            check=False,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True  # Return stdout/stderr as strings
        )
        if completed_process.returncode != 0:
            message = (
                f"An error occurred while running laptops_final.py.\n"
                f"Return code: {completed_process.returncode}\n"
                f"Error Output: {completed_process.stderr}"
            )
            success = False
        else:
            message = "Master File executed successfully."
            success = True
    except Exception as e:
        message = "Unexpected error: " + str(e)
        success = False

    return render_template("welcome.html", message=message, success=success)



'''# Run Flask in a background thread
def run_app():
    app.run(port=5000, debug=False)

# Start Flask in a separate thread
thread = threading.Thread(target=run_app)
thread.start()'''
if __name__ == "__main__":
    app.run(port=5000, debug=False)
